@@ -31,6 +31,7 @@ starrydb module provide functions of a NoSQL database for lightweight data.
 """
 
 import os
+import sys
 import time
 import shutil
 import inspect
@@ -46,7 +47,7 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.6.8"
+VERSION = "0.6.9"
 LOCK_SH = 0
 LOCK_EX = 1
 RUN_PATH = "/run/starrydb/"
@@ -74,10 +75,23 @@ class StarryLog(object):
     def __init__(self, file=None, lvl=logging.NOTSET):
         self.logger = None
 
-        if pathlib.Path(file).is_file():
+        if file is not None:
+            if pathlib.Path(file).is_file():
+                try:
+                    logging.basicConfig(
+                        handlers=[logging.FileHandler(file)],
+                        format=f"%(asctime)s <%(levelname)s> {inspect.stack()[-1][1].split('/')[-1]}[%(process)d]: %(message)s",
+                        level=lvl)
+
+                    logging.raiseExceptions = False
+
+                    self.logger = logging.getLogger()
+                except:
+                    pass
+        else:
             try:
                 logging.basicConfig(
-                    handlers=[logging.FileHandler(file)],
+                    stream=sys.stdout,
                     format=f"%(asctime)s <%(levelname)s> {inspect.stack()[-1][1].split('/')[-1]}[%(process)d]: %(message)s",
                     level=lvl)
 
@@ -181,12 +195,11 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, address=SERVER_ADDRESS, depots=[], cache_address=CACHE_SOCKET, cache_size=0, log_file=None, rw_ratio=5):
+    def __init__(self, address=SERVER_ADDRESS, depots=[], cache_size=0, log_file=None, log_level="WARNING", rw_ratio=5):
         self.address = address
-        self.cache = cache_address
         self.cursor = 0
         self.depots = {}
-        self.logger = StarryLog(log_file) if log_file is not None else None
+        self.logger = StarryLog(log_file, eval(f'logging.{log_level}'))
         self.rw_ratio = rw_ratio
         self.memcached = None
 
@@ -194,16 +207,16 @@ class StarryServer(object):
 
         if int(cache_size) > 0:
             try:
-                subprocess.check_output("kill -9 `ps axf | grep " + self.cache + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output("kill -9 `ps axf | grep " + CACHE_SOCKET + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
             try:
-                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache_size)} -s {self.cache}")
+                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache_size)} -s {CACHE_SOCKET}")
 
                 time.sleep(1)
 
-                self.memcached = LibMemcachedClient(self.cache)
+                self.memcached = LibMemcachedClient(CACHE_SOCKET)
             except Exception as e:
                 self.logger.error(f"Caught exception: {e.__doc__}({e})")
 
@@ -221,8 +234,8 @@ class StarryServer(object):
                 if depot.get("cache", False) is True:
                     self.depots[depot_path].enableCache()
 
-                if depot.get("flush", False) is True:
-                    self.depots[depot_path].enableFlush()
+                if depot.get("fsync", False) is True:
+                    self.depots[depot_path].enableFsync()
 
                 self.cursor += 1
 
@@ -236,7 +249,7 @@ class StarryServer(object):
                 pass
 
             try:
-                subprocess.check_output("kill -9 `ps axf | grep " + self.cache + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output("kill -9 `ps axf | grep " + CACHE_SOCKET + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
@@ -287,7 +300,7 @@ class StarryDepot(object):
         self.index = index
         self.memcached = memcached
         self.cache = None
-        self.flush = False
+        self.fsync = False
         self.bak = False
         self.thread_cond = threading.Condition()
         self.thread_lock = threading.RLock()
@@ -349,13 +362,13 @@ class StarryDepot(object):
         except:
             pass
 
-    def enableFlush(self):
-        self.flush = True
+    def enableFsync(self):
+        self.fsync = True
 
         return (True, "")
 
-    def disableFlush(self):
-        self.flush = False
+    def disableFsync(self):
+        self.fsync = False
 
         return (True, "")
 
@@ -777,7 +790,7 @@ class StarryDepot(object):
 
                 return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value, self.flush)
+        (ret, msg) = _writeFile(file, value, self.fsync)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -838,7 +851,7 @@ class StarryDepot(object):
 
                 return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value, self.flush)
+        (ret, msg) = _writeFile(file, value, self.fsync)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -931,7 +944,7 @@ class StarryDepot(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value, self.flush)
+        (ret, msg) = _writeFile(file, value, self.fsync)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -963,7 +976,7 @@ class StarryDepot(object):
         if path is None:
             path = self.database.joinpath(".backup")
 
-        return _copyFile(file, path.joinpath(f"{key}.mpb"), self.flush)
+        return _copyFile(file, path.joinpath(f"{key}.mpb"), self.fsync)
 
     def _recover(self, key):
         backup_file = self.database.joinpath(".backup", f"{key}.mpb")
@@ -992,7 +1005,7 @@ class StarryDepot(object):
 
             return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _copyFile(backup_file, self.database.joinpath(f"{key}.mpb"), self.flush)
+        (ret, msg) = _copyFile(backup_file, self.database.joinpath(f"{key}.mpb"), self.fsync)
         if ret is False:
             return (ret, msg)
 
@@ -1024,7 +1037,7 @@ class StarryDepot(object):
                     if ret is False:
                         return (ret, msg)
 
-        (ret, msg) = _copyFile(source_file, file, self.flush)
+        (ret, msg) = _copyFile(source_file, file, self.fsync)
         if ret is False:
             if not backup_file.exists():
                 try:
@@ -1367,13 +1380,13 @@ def _readFile(src):
     except Exception as e:
         return (False, f"Caught exception: {e.__doc__}({e})")
 
-def _writeFile(dst, value, flush=False):
+def _writeFile(dst, value, fsync=False):
     try:
         with dst.open(mode="wb", buffering=0) as descriptor:
             descriptor.write(value)
             descriptor.truncate()
 
-            if flush is True:
+            if fsync is True:
                 descriptor.flush()
                 os.fdatasync(descriptor.fileno())
 
@@ -1384,7 +1397,7 @@ def _writeFile(dst, value, flush=False):
 
     return (True, "")
 
-def _copyFile(src, dst, flush=False):
+def _copyFile(src, dst, fsync=False):
     (ret, value) = _readFile(src)
     if ret is False:
         return (ret, value)
@@ -1392,7 +1405,7 @@ def _copyFile(src, dst, flush=False):
     if dst.exists():
         tmp_path = dst.with_suffix(".mpb.tmp")
 
-        (ret, msg) = _writeFile(tmp_path, value, flush)
+        (ret, msg) = _writeFile(tmp_path, value, fsync)
         if ret is False:
             try:
                 tmp_path.unlink()
@@ -1409,11 +1422,11 @@ def _copyFile(src, dst, flush=False):
             except:
                 pass
 
-            (ret, msg) = _writeFile(dst, value, flush)
+            (ret, msg) = _writeFile(dst, value, fsync)
             if ret is False:
                 return (ret, msg)
     else:
-        (ret, msg) = _writeFile(dst, value, flush)
+        (ret, msg) = _writeFile(dst, value, fsync)
         if ret is False:
             try:
                 dst.unlink()
